@@ -30,6 +30,7 @@
 #include <QtCore>
 #include <QtGui>
 #include "Module.hpp"
+#include "MemPtr.hpp"
 
 static std::string helppath;
   
@@ -51,6 +52,33 @@ void Tokenize(const std::string& str, std::vector<std::string>& tokens,
     }
 }
 
+//!
+//@Module JITCONTROL Control the Just In Time Compiler
+//@@Section FREEMAT
+//@@Usage
+//The @|jitcontrol| functionality in FreeMat allows you to control
+//the use of the Just In Time (JIT) compiler.  
+//!
+ArrayVector JITControlFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
+  if (arg.size() < 1) {
+    if (eval->JITControl())
+      return singleArrayVector(Array::stringConstructor("on"));
+    else
+      return singleArrayVector(Array::stringConstructor("off"));
+  } else {
+    if (!arg[0].isString())
+      throw Exception("jitcontrol function takes only a single, string argument");
+    string txt = arg[0].getContentsAsStringUpper();
+    if (txt == "ON")
+      eval->JITControl(true);
+    else if (txt == "OFF")
+      eval->JITControl(false);
+    else
+      throw Exception("jitcontrol function argument needs to be 'on/off'");
+  }
+  return ArrayVector();
+}
+
 //!
 //@Module DBAUTO Control Dbauto Functionality
 //@@Section DEBUG
@@ -138,27 +166,6 @@ ArrayVector HelpWinFunction(int nargout, const ArrayVector& arg, Interpreter* ev
   return ArrayVector();
 }
 
-//!
-//@Module EDITOR Open Editor Window
-//@@Section FREEMAT
-//@@Usage
-//Brings up the editor window.  The @|editor| function takes no
-//arguments:
-//@[
-//  editor
-//@]
-//!
-ArrayVector EditorFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
-  static FMEditor *edit = NULL;
-  if (edit == NULL) {
-    edit = new FMEditor(eval);
-    QObject::connect(eval,SIGNAL(RefreshBPLists()),edit,SLOT(RefreshBPLists()));
-    QObject::connect(eval,SIGNAL(ShowActiveLine()),edit,SLOT(ShowActiveLine()));
-  }
-  edit->showNormal();
-  edit->raise();
-  return ArrayVector();
-}
 
 ArrayVector EndFunction(int nargout, const ArrayVector& arg) {
   if (arg.size() != 3)
@@ -190,7 +197,172 @@ ArrayVector PathToolFunction(int nargout, const ArrayVector& arg, Interpreter* e
   eval->rescanPath();
   return ArrayVector();
 }
-  
+
+//convert the supplied string to a decimal integer
+static double hex2dec_helper (string t) {
+  return (double) strtoll(t.c_str(),NULL,16);
+}
+
+//!
+//@Module HEX2DEC Convert Hexadecimal Numbers To Decimal
+//@@Section ELEMENTARY
+//@@Usage
+//Converts a hexadecimal number (encoded as a string matrix) into integers.
+//The syntax for its use is
+//@[
+//   y = hex2dec(x)
+//@]
+//where @|x| is a character matrix where each row represents an integer
+//in hexadecimal form.  The output is of type @|FM_DOUBLE|.
+//@@Examples
+//@<
+//hex2dec('3ff')
+//@>
+//Or for a more complex example
+//@<
+//hex2dec(['0ff';'2de';'123'])
+//@>
+//!
+ArrayVector Hex2DecFunction(int nargout, const ArrayVector& arg) {
+  if (arg.size() == 0)
+    throw Exception("hex2dec requires an argument");
+  Array x(arg[0]);
+  if (x.dataClass() != FM_STRING)
+    throw Exception("hex2dec argument must be a string");
+  if (x.isVector()) {
+    string str(ArrayToString(x));
+    return ArrayVector() << Array::doubleConstructor(hex2dec_helper(str));
+  } else {
+    // Create a buffer to hold the string
+    int numrows = x.rows();
+    int numcolumns = x.columns();
+    MemBlock<uint8> p(numcolumns+1);
+    uint8 *b = &p;
+    // How many strings to convert?
+    const uint8* dp = (const uint8*) x.getDataPointer();
+    double *qp = (double*) Array::allocateArray(FM_DOUBLE,numrows);
+    for (int i=0;i<numrows;i++) {
+      for (int j=0;j<numcolumns;j++)
+	b[j] = dp[i+j*numrows];
+      qp[i] = hex2dec_helper((char*)b);
+    }
+    return ArrayVector() << Array(FM_DOUBLE,Dimensions(numrows,1),qp);
+  }
+  return ArrayVector();
+}
+
+//!
+//@Module DEC2HEX Convert Decimal Number to Hexadecimal
+//@@Section ELEMENTARY
+//@@Usage
+//Converts an integer value into its hexadecimal representation.  The syntax
+//for its use is
+//@[
+//   y = dec2hex(x)
+//@]
+//where @|x| is an integer (and is promoted to a 64-bit integer if it is not).
+//The returned value @|y| is a string containing the hexadecimal representation
+//of that integer.  If you require a minimum length for the hexadecimal
+//representation, you can specify an optional second argument
+//@[
+//   y = dec2hex(x,n)
+//@]
+//where @|n| indicates the minimum number of digits in the representation.
+//@@Example
+//Here are some simple examples:
+//@<
+//dec2hex(1023)
+//@>
+//@<
+//dec2hex(58128493)
+//@>
+//@@Tests
+//@$"y=dec2hex(1023)","'3ff'","exact"
+//@$"y=dec2hex(16,4)","'0010'","exact"
+//!
+ArrayVector Dec2HexFunction(int nargout, const ArrayVector& arg) {
+  char buffer[1000];
+  char fbuffer[1000];
+  if (arg.size() < 1)
+    throw Exception("dec2hex requires at least one argument");
+  Array x(arg[0]);
+  if (!x.isScalar())
+    throw Exception("dec2hex requires first argument to be a scalar");
+  x.promoteType(FM_INT64);
+  const int64 *dp = (const int64*) x.getDataPointer();
+  int64 xval = dp[0];
+  sprintf(buffer,"%llx",xval);
+  string retString(buffer);
+  if (arg.size() == 1) {
+    return ArrayVector() << Array::stringConstructor(retString);
+  } else {
+    int n = ArrayToInt32(arg[1]);
+    if ((n < 1) || (n > 32))
+      throw Exception("illegal number of digits requested in dec2hex function");
+    while (retString.size() < n)
+      retString = '0' + retString;
+    return ArrayVector() << Array::stringConstructor(retString);
+  }
+  return ArrayVector();
+}
+
+
+//!
+//@Module NUM2HEX Convert Numbers to IEEE Hex Strings
+//@@Section ELEMENTARY
+//@@Usage
+//Converts single and double precision arrays to IEEE hex strings.  The
+//syntax for its use is
+//@[
+//   y = num2hex(x)
+//@]
+//where @|x| is either a @|float| or @|double| array.  The output @|y| is
+//a @|n-by-p| character array, where @|n| is the number of elements in @|x|,
+//and @|p| is 16 for @|double| arrays, and 8 for @|single| arrays.
+//@@Example
+//Some interesting numbers
+//@<
+//num2hex([1 0 0.1 -pi inf nan])
+//@>
+//The same in single precision
+//@<
+//num2hex(float([1 0 0.1 -pi inf nan]))
+//@>
+//@@Tests
+//@$"y=num2hex(-pi)","'c00921fb54442d18'","exact"
+//@$"y=num2hex(.1f)","'3dcccccd'","exact"
+//!
+ArrayVector Num2HexFunction(int nargout, const ArrayVector& arg) {
+  if (arg.size() != 1)
+    throw Exception("num2hex fucntion requires a single argument");
+  Array x(arg[0]);
+  if ((x.dataClass() != FM_FLOAT) && (x.dataClass() != FM_DOUBLE))
+    throw Exception("num2hex only works on float and double arrays");
+  int len(x.getLength());
+  if (x.dataClass() == FM_FLOAT) {
+    uint8* dp = (uint8*) Array::allocateArray(FM_STRING,len*8);
+    const float *xp = (const float *) x.getDataPointer();
+    for (int i=0;i<len;i++) {
+      char buffer[10];
+      ToHexString(xp[i],buffer);
+      for (int j=0;j<8;j++)
+	dp[i+len*j] = buffer[j];
+    }
+    return ArrayVector() << Array(FM_STRING,Dimensions(len,8),dp,false);
+  } else {
+    uint8* dp = (uint8*) Array::allocateArray(FM_STRING,len*16);
+    const double *xp = (const double *) x.getDataPointer();
+    for (int i=0;i<len;i++) {
+      char buffer[18];
+      ToHexString(xp[i],buffer);
+      for (int j=0;j<16;j++)
+	dp[i+len*j] = buffer[j];
+    }
+    return ArrayVector() << Array(FM_STRING,Dimensions(len,16),dp,false);
+  }
+  return ArrayVector();
+}
+
 //!
 //@Module HELP Help
 //@@Section FREEMAT
@@ -331,30 +503,28 @@ void ClearVariable(Interpreter* eval, string name) {
 
 void ClearAllFunction(Interpreter* eval) {
   ClearLibs(eval);
-  stringVector names = eval->getContext()->getCurrentScope()->listAllVariables();
+  stringVector names = eval->getContext()->listAllVariables();
   for (int i=0;i<names.size();i++)
     ClearVariable(eval,names[i]);
 }
 
 void ClearPersistent(Interpreter* eval) {
-  stringVector names = eval->getContext()->getGlobalScope()->listAllVariables();
+  stringVector names = eval->getContext()->listGlobalVariables();
   for (int i=0;i<names.size();i++) {
     if ((names[i].size() >= 1) && (names[i][0] == '_'))
-      eval->getContext()->getGlobalScope()->deleteVariable(names[i]);
+      eval->getContext()->deleteGlobalVariable(names[i]);
   }
-  eval->getContext()->getGlobalScope()->clearPersistentVariableList();
-  eval->getContext()->getCurrentScope()->clearPersistentVariableList();
+  eval->getContext()->clearPersistentVariableList();
 }
 
 void ClearGlobal(Interpreter* eval) {
-  stringVector names = eval->getContext()->getGlobalScope()->listAllVariables();
+  stringVector names = eval->getContext()->listGlobalVariables();
   for (int i=0;i<names.size();i++) {
     if ((names[i].size() >= 1) && (names[i][0] != '_')) {
-      eval->getContext()->getGlobalScope()->deleteVariable(names[i]);
+      eval->getContext()->deleteGlobalVariable(names[i]);
     }
   }
-  eval->getContext()->getGlobalScope()->clearGlobalVariableList();
-  eval->getContext()->getCurrentScope()->clearGlobalVariableList();
+  eval->getContext()->clearGlobalVariableList();
 }
 
 ArrayVector ClearFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
@@ -427,7 +597,7 @@ ArrayVector WhoFunction(int nargout, const ArrayVector& arg, Interpreter* eval)
   stringVector names;
   char buffer[1000];
   if (arg.size() == 0) {
-    names = eval->getContext()->getCurrentScope()->listAllVariables();
+    names = eval->getContext()->listAllVariables();
   } else {
     for (i=0;i<arg.size();i++) {
       Array varName(arg[i]);
@@ -624,10 +794,7 @@ ArrayVector SizeFunction(int nargout, const ArrayVector& arg) {
   if (arg.size() < 1)
     throw Exception("size function requires either one or two arguments");
   Dimensions sze;
-  if (arg[0].isEmpty()) {
-    sze = Dimensions(0,0);
-  } else
-    sze = arg[0].dimensions();
+  sze = arg[0].dimensions();
   if (arg.size() == 1) {
     if (nargout > 1) {
       ArrayVector retval;
@@ -1144,7 +1311,7 @@ ArrayVector WhichFunction(int nargout, const ArrayVector& arg, Interpreter* eval
   string fname = arg[0].getContentsAsString();
   bool isFun;
   FuncPtr val;
-  isFun = eval->getContext()->lookupFunction(fname,val);
+  isFun = eval->lookupFunction(fname,val);
   char buffer[1000];
   Array ret(Array::emptyConstructor());
   if (isFun) {
