@@ -20,6 +20,8 @@
 #ifndef __Utils_hpp__
 #define __Utils_hpp__
 
+#include "Types.hpp"
+
 float complex_abs(float, float);
 double complex_abs(double, double);
 void c_log(float real, float imag, float *re, float *im);
@@ -30,4 +32,6 @@ void z_log(double real, double imag, double *re, double *im);
 void z_exp(double real, double imag, double *re, double *im);
 void z_sqrt(double real, double imag, double *re, double *im);
 void z_sqr(double real, double imag, double *re, double *im);
+
+bool contains(rvstring& list, std::string s, bool regexpmode);
 #endif
