@@ -28,7 +28,7 @@
 #include "Data.hpp"
 #include "RefVec.hpp"
 #include <QVector>
-#include <QList>
+#include "List.hpp"
 #include <QStringList>
 #include <QSharedDataPointer>
 
@@ -565,6 +565,11 @@ public:
    * specified dimenstions.
    */
   static Array doubleMatrixConstructor(int rows, int cols);
+  /**
+   * Double complex matrix constructor - Construct an FM_DCOMPLEX object 
+   * with the specified dimenstions.
+   */  
+  static Array dcomplexMatrixConstructor(int rows, int cols);
   /**
    * Complex constructor - Construct an FM_COMPLEX object with a 
    * complex scalar value.
@@ -777,15 +782,6 @@ public:
    * Throws an exception if the argument contains more than one non-colon index
    */
   void deleteNDimSubset(ArrayVector& args, Interpreter* m_eval);
-  /**
-   * Summarize this array when it appears in a Cell array.
-   */
-  void summarizeCellEntry(Interpreter* eval) const;
-  /**
-   * Print some reasonable representation of this array to the
-   * the supplied stream.
-   */
-  //    void printMe(int printLimit, int termWidth = 80) const;
   /**
    * Return the number of rows.
    */
