@@ -118,6 +118,10 @@ class Interpreter : public QThread {
    * autostop storage flag
    */
   bool autostop;
+  /**
+   * jit control flag
+   */
+  bool jitcontrol;
   /**
    * When this flag is active, autostop does nothing.
    */
@@ -247,6 +251,10 @@ class Interpreter : public QThread {
    * The capture state
    */
   bool m_captureState;
+  /**
+   * The quiet level
+   */
+  int m_quietlevel;
   /******************************************
    *  Public Methods for the Interpreter    *
    ******************************************/
@@ -279,6 +287,11 @@ public:
     m_threadNargout = nargout;
     m_threadErrorState = false;
   }
+  /**
+   * Manipulate the quiet level
+   */
+  inline int getQuietLevel() {return m_quietlevel;}
+  inline void setQuietLevel(int t) {m_quietlevel = t;}
   /**
    * Manipulate the diary state
    */
@@ -360,6 +373,7 @@ public:
    * Output the following text message.
    */
   void outputMessage(string msg);
+  void outputMessage(const char* format,...);
   /**
    * Output the following error message.
    */
@@ -393,7 +407,7 @@ public:
   /**
    * Get the context for the interface.
    */
-  Context* getContext();
+  inline Context* getContext() {return context;}
   /**
    * Refresh the breakpoints in the code.  Call this after updating
    * code for functions, or breakpoints will not work.
@@ -458,8 +472,13 @@ public:
    * Set the autostop flag - this flag determines what happens when
    * an exception occurs
    */
-  bool AutoStop();
-  void AutoStop(bool a);
+  inline bool AutoStop() {return autostop;}
+  inline void AutoStop(bool a) {autostop = a;}
+  /**
+   * Set the JITControl flag
+   */
+  inline bool JITControl() {return jitcontrol;}
+  inline void JITControl(bool a) {jitcontrol = a;}
   /**
    * Set the print limit (number of element printed prior to truncation).
    */
@@ -536,6 +555,10 @@ signals:
    * Send a string of text to a terminal somewhere
    */
   void outputRawText(string);
+  /**
+   * Flush the I/O buffers
+   */ 
+  void Flush();
   /**
    * User has changed the current working directory
    */
@@ -601,6 +624,10 @@ private:
   void handlePassByReference(const tree &q, stringVector arguments,
 			     ArrayVector m,stringVector keywords,
 			     treeVector keyexpr, int* argTypeMap);
+  /**
+   * Create a variable in the correct scope, and return a reference to it
+   */
+  ArrayReference createVariable(string name);
   /**
    * Perform a binary operator with the given name
    */
@@ -609,6 +636,10 @@ private:
    * Perform a unary operator with the given name
    */
   Array DoUnaryOperator(const tree &t, UnaryFunc fnc, string fname);
+  /**
+   * Handle the construction of a function pointer
+   */
+  Array FunctionPointer(const tree &args);
   /**
    * Dispatch a function pointer
    */
@@ -1024,6 +1055,7 @@ private:
   friend ArrayVector PCodeFunction(int, const ArrayVector&, Interpreter*);
   friend class MFunctionDef;
   friend class ImportedFunctionDef;
+  friend class AnonymousFunctionDef;
 };
 
 void sigInterrupt(int arg);
