@@ -32,7 +32,7 @@ string fm_reserved[22] = {
 int fm_reserved_count = sizeof(fm_reserved)/sizeof(fm_reserved[0]);
 
 Token::Token(byte tok, unsigned pos, string text) :
-  m_tok(tok), m_pos(pos), m_text(text), m_bpflag(false) {
+  m_tok(tok), m_pos(pos), m_text(text) {
 }
 
 Token::Token() {
@@ -66,11 +66,7 @@ bool Token::IsRightAssociative() const {
 
 ostream& operator<<(ostream& o, const Token& b) {
   o << TokenToString(b) << " (" << (b.Position() >> 16)
-    << "," << (b.Position() & 0xffff);
-  if (b.BPFlag())
-    o << "," << "BP)\r\n";
-  else
-    o << ")\r\n";
+    << "," << (b.Position() & 0xffff) << ")\r\n";
   return o;
 }
 
@@ -139,6 +135,8 @@ string TokenToString(const Token& b) {
   case TOK_FUNCTION_DEFS: return "functions:";
   case TOK_SCRIPT: return "script:";
   case TOK_DBTRACE: return "dbtrace";
+  case TOK_ANONYMOUS_FUNC: return "anon func";
+  case TOK_NEST_FUNC: return "nest func";
   }
   return string(1,(char) b.Value());
 }
@@ -147,7 +145,6 @@ void FreezeToken(const Token& a, Serialize *s) {
   s->putByte(a.m_tok);
   s->putInt(a.m_pos);
   s->putString(a.m_text.c_str());
-  s->putBool(a.m_bpflag);
 }
 
 Token ThawToken(Serialize *s) {
@@ -155,7 +152,6 @@ Token ThawToken(Serialize *s) {
   a.m_tok = s->getByte();
   a.m_pos = s->getInt();
   a.m_text = s->getString();
-  a.m_bpflag = s->getBool();
   a.FillArray();
   return a;
 }
@@ -164,14 +160,14 @@ void Token::FillArray() {
   Array retval;
   switch(m_tok) {
   case TOK_INTEGER:
-    int iv;
+    long iv;
     double fv;
     iv = strtol(m_text.c_str(),NULL,10);
     if ((errno == ERANGE) && ((iv == LONG_MAX) || (iv == LONG_MIN))) {
       fv = strtod(m_text.c_str(),NULL);
       retval = Array::doubleConstructor(fv);
     } else {
-      retval = Array::int32Constructor(iv);
+      retval = Array::int32Constructor((int32)iv);
     }
     break;
   case TOK_FLOAT:
