@@ -27,6 +27,138 @@
 #include "QTRenderEngine.hpp"
 #include "HandleCommands.hpp"
 #include <qgl.h>
+#include <QList>
+
+QList<double> GetTicksOuter(double amin, double amax, bool isLog, int requestedCounts) {
+  double arange = amax - amin;
+  double astep = pow(10.0,floor(log10(arange)));
+  double nsteps = arange/astep;
+  double aval;
+  QList<double> retvec;
+  if (requestedCounts >= 10) {
+    if (nsteps <= 1)
+      astep /= 10.0;
+    else if (nsteps <= 2)
+      astep /= 5.0;
+    else if (nsteps <= 5)
+      astep /= 2.0;
+  } else if (requestedCounts >= 5) {
+    if (nsteps > 5) {
+      astep *= 2.0;
+      nsteps = arange/astep;
+    }
+    if (nsteps <= 1)
+      astep /= 5.0;
+  } else if (requestedCounts >= 2) {
+    if (nsteps > 2) {
+      astep *= 5.0;
+      nsteps = arange/astep;
+    }
+    if (nsteps <= 1)
+      astep /= 2.0;
+  }
+  if (isLog)
+    astep = ceil(astep);
+  if ((amin < 0) && (amax > 0)) {
+    aval = 0;
+    while (aval < amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+    retvec.push_back(aval);
+    aval = -astep;
+    while (aval > amin) {
+      retvec.push_front(aval);
+      aval -= astep;
+    }
+    retvec.push_front(aval);
+  } else if ((amin == 0) && (amax > 0)) {
+    aval = 0;
+    while (aval < amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+    retvec.push_back(aval);
+  } else if ((amin < 0) && (amax == 0)) {
+    aval = 0;
+    while (aval > amin) {
+      retvec.push_front(aval);
+      aval -= astep;
+    }
+    retvec.push_front(aval);
+  } else {
+    aval = floor(amin/astep)*astep;
+    while (aval < amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+    retvec.push_back(aval);
+  }
+  return retvec;
+}
+
+QList<double> GetTicksInner(double amin, double amax, bool isLog, int requestedCounts) {
+  double arange = amax - amin;
+  double astep = pow(10.0,floor(log10(arange)));
+  double nsteps = arange/astep;
+  double aval;
+  QList<double> retvec;
+  if (requestedCounts >= 10) {
+    if (nsteps <= 1)
+      astep /= 10.0;
+    else if (nsteps <= 2)
+      astep /= 5.0;
+    else if (nsteps <= 5)
+      astep /= 2.0;
+  } else if (requestedCounts >= 5) {
+    if (nsteps > 5) {
+      astep *= 2.0;
+      nsteps = arange/astep;
+    }
+    if (nsteps <= 1)
+      astep /= 5.0;
+  } else if (requestedCounts >= 2) {
+    if (nsteps > 2) {
+      astep *= 5.0;
+      nsteps = arange/astep;
+    }
+    if (nsteps <= 1)
+      astep /= 2.0;
+  }
+  if (isLog) astep = ceil(astep);
+  if ((amin < 0) && (amax > 0)) {
+    aval = 0;
+    while (aval <= amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+    aval = -astep;
+    while (aval >= amin) {
+      retvec.push_front(aval);
+      aval -= astep;
+    }
+  } else if ((amin == 0) && (amax > 0)) {
+    aval = 0;
+    while (aval <= amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+  } else if ((amin < 0) && (amax == 0)) {
+    aval = 0;
+    while (aval >= amin) {
+      retvec.push_front(aval);
+      aval -= astep;
+    }
+  } else {
+    aval = ceil(amin/astep)*astep;
+    while (aval <= amax) {
+      retvec.push_back(aval);
+      aval += astep;
+    }
+  }
+  return retvec;
+}
+
 
 // Property list & status
 //    activepositionproperty
@@ -223,27 +355,18 @@ void FormatAxisManual(double t1, double t2, int tickcount,
 		      double& tStart, double &tStop,
 		      std::vector<double> &tickLocations,
 		      std::vector<std::string> &tlabels) {
-  double tBegin, tEnd;
-  double delt = (t2-t1)/tickcount;
-  int n = (int) ceil(log10(delt));
-  double rdelt = delt/pow(10.0,(double)n);
-  int p = (int) floor(log(rdelt)/M_LN2);
-  double tDelt = pow(10.0,(double) n)*pow(2.0,(double) p);
-  if (isLogarithmic)
-    tDelt = ceil(tDelt);
-  tStart = t1;
-  tStop = t2;
-  tBegin = tDelt*ceil(t1/tDelt);
-  tEnd = floor(t2/tDelt)*tDelt;
-  Array trange(Array::doubleRangeConstructor(tBegin,tDelt,tEnd,false));
-  int tCount = trange.getLength();
+  int tCount;
   tickLocations.clear();
   tlabels.clear();
   bool exponentialForm;
   exponentialForm = false;
-  const double *dp = (const double *) trange.getDataPointer();
+  QList<double> tick_locations(GetTicksInner(t1,t2,isLogarithmic,tickcount));
+  if (tick_locations.empty()) return;
+  tStart = tick_locations.front();
+  tStop = tick_locations.back();
+  tCount = tick_locations.size();
   for (int i=0;i<tCount;i++) {
-    double tloc = dp[i];
+    double tloc = tick_locations[i];
     if (!isLogarithmic)
       tickLocations.push_back(tloc);
     else
@@ -252,7 +375,7 @@ void FormatAxisManual(double t1, double t2, int tickcount,
       exponentialForm |= (fabs(log10(fabs(tloc))) >= 4.0);
   }
   for (int i=0;i<tCount;i++) {
-    double tloc = tBegin+i*tDelt;
+    double tloc = tick_locations[i];
     if (!isLogarithmic)
       tlabels.push_back(TrimPrint(tloc,exponentialForm));
     else
@@ -265,40 +388,19 @@ void FormatAxisAuto(double tMin, double tMax, int tickcount,
 		    double& tStart, double &tStop,
 		    std::vector<double> &tickLocations,
 		    std::vector<std::string> &tlabels) {
-  //  bool integerMode = false;
-  double tBegin, tEnd;
-  double delt = (tMax-tMin)/(tickcount);
-  int n = (int) ceil(log10(delt));
-  double rdelt = delt/pow(10.0,(double)n);
-  int p = (int) floor(log(rdelt)/M_LN2);
-  double tDelt = pow(10.0,(double) n)*pow(2.0,(double) p);
-  if (isLogarithmic) 
-    tDelt = ceil(tDelt);
-  tStart = floor(tMin/tDelt)*tDelt;
-  tStop = ceil(tMax/tDelt)*tDelt;
-  // Recheck for integer limits...
-  //   qDebug("tickcount = %d",tickcount);
-  //   if ((tMin == rint(tMin)) && (tMax == rint(tMax)) && (!isLogarithmic)) {
-  //     tStart = tMin;
-  //     tStop = tMax;
-  //     if ((tickcount % 2) == 1)
-  //       tickcount++;
-  //     tDelt = (tStop - tStart)/(tickcount);
-  //     if ((tMax-tMin) > 1) integerMode = true;
-  //   }
-  tBegin = tStart;
-  tEnd = tStop;
-  Array trange(Array::doubleRangeConstructor(tBegin,tDelt,tEnd,false));
-  int tCount = trange.getLength();
+  int tCount;
   tickLocations.clear();
   tlabels.clear();
   bool exponentialForm;
   exponentialForm = false;
-  const double *dp = (const double *) trange.getDataPointer();
-  //    qDebug("Format %f %f %d %d",tMin,tMax,tickcount,tCount);
+  //  const double *dp = (const double *) trange.getDataPointer();
+  QList<double> tick_locations(GetTicksOuter(tMin,tMax,isLogarithmic,tickcount));
+  if (tick_locations.empty()) return;
+  tStart = tick_locations.front();
+  tStop = tick_locations.back();
+  tCount = tick_locations.size();
   for (int i=0;i<tCount;i++) {
-    double tloc = dp[i];
-    //      qDebug("  Point %f",tloc);
+    double tloc = tick_locations[i];
     if (!isLogarithmic)
       tickLocations.push_back(tloc);
     else
@@ -307,7 +409,7 @@ void FormatAxisAuto(double tMin, double tMax, int tickcount,
       exponentialForm |= (fabs(log10(fabs(tloc))) >= 4.0);
   }
   for (int i=0;i<tCount;i++) {
-    double tloc = tBegin+i*tDelt;
+    double tloc = tick_locations[i];
     if (!isLogarithmic)
       tlabels.push_back(TrimPrint(tloc,exponentialForm));
     else
@@ -389,7 +491,7 @@ void HandleAxis::ConstructProperties() {
   // for the axes.  Defaults to white.
   //  \item @|colororder| - @|color vector| - A vector of color specs (in 
   // RGB) that are cycled between when drawing line plots into this axis.
-  // The default is order red, green, blue, yellow, magenta, cyan, black.
+  // The default is order blue,green,red,cyan,magenta,yellow,black.
   //  \item @|datalimits| - @|six vector| - A vector that contains the x, y and z
   // limits of the data for children of the current axis.  Changes to this
   // property are ignored - it is calculated by FreeMat based on the datasets.
@@ -681,13 +783,13 @@ void HandleAxis::SetupDefaults() {
   SetThreeVectorDefault("color",1,1,1);
   // Set up the default color order
   std::vector<double> colors;
-  colors.push_back(1.0); colors.push_back(0.0); colors.push_back(0.0); 
-  colors.push_back(0.0); colors.push_back(1.0); colors.push_back(0.0); 
-  colors.push_back(0.0); colors.push_back(0.0); colors.push_back(1.0);
-  colors.push_back(1.0); colors.push_back(1.0); colors.push_back(0.0);
-  colors.push_back(1.0); colors.push_back(0.0); colors.push_back(1.0);
-  colors.push_back(0.0); colors.push_back(1.0); colors.push_back(1.0);
-  colors.push_back(0.0); colors.push_back(0.0); colors.push_back(0.0);
+  colors.push_back(0.0); colors.push_back(0.0); colors.push_back(1.0); 
+  colors.push_back(0.0); colors.push_back(0.5); colors.push_back(0.0); 
+  colors.push_back(1.0); colors.push_back(0.0); colors.push_back(0.0);
+  colors.push_back(0.0); colors.push_back(.75); colors.push_back(.75);
+  colors.push_back(.75); colors.push_back(0.0); colors.push_back(.75);
+  colors.push_back(.75); colors.push_back(.75); colors.push_back(0.0);
+  colors.push_back(.25); colors.push_back(.25); colors.push_back(.25);
   HPVector *hp = (HPVector*) LookupProperty("colororder");
   hp->Data(colors);
   SetThreeVectorDefault("dataaspectratio",1,1,1);
@@ -1032,6 +1134,7 @@ void HandleAxis::DrawBox(RenderEngine &gc) {
 	   limits[1], limits[3], limits[4],
 	   limits[1], limits[3], limits[5],
 	   limits[0], limits[3], limits[5]);
+  
   gc.depth(true);
 }
 
@@ -1540,7 +1643,7 @@ int HandleAxis::GetTickCount(RenderEngine &gc,
   gc.toPixels(x2,y2,z2,u2,v2);
   double axlen;
   axlen = sqrt((u2-u1)*(u2-u1) + (v2-v1)*(v2-v1));
-  int numtics = (int)(qMax(2.0,axlen/100.0));
+  int numtics = (int)(qMax(2.0,axlen/25.0));
   return numtics;
 }
 
@@ -1764,9 +1867,11 @@ void HandleAxis::RePackFigure() {
   posheight = posheight/height;
   posx0 = (posx0+outerpos[0])/width;
   posy0 = (posy0+outerpos[1])/height;
+  poswidth = qMax(0.0,poswidth);
+  posheight = qMax(0.0,posheight);
   HPFourVector *hp = (HPFourVector*) LookupProperty("position");
   hp->Value(posx0,posy0,poswidth,posheight);
-  //    qDebug("Pack %f %f %f %f",posx0,posy0,poswidth,posheight);
+  //  qDebug("Pack %f %f %f %f",posx0,posy0,poswidth,posheight);
 }
 
 void HandleAxis::UpdateLimits(bool x, bool y, bool z, bool a, bool c) {
@@ -2424,6 +2529,8 @@ void HandleAxis::PaintMe(RenderEngine& gc) {
     DrawGridLines(gc);
     DrawMinorGridLines(gc);
   }
+  std::vector<double> limits(GetAxisLimits());
+  gc.setClipBox(limits);
   DrawChildren(gc);
   if (StringCheck("visible","on")) {
     DrawAxisLines(gc);
