@@ -29,6 +29,15 @@
 #include "HandleFigure.hpp"
 #include "RenderEngine.hpp"
 #include "Context.hpp"
+#include <QPainterPath>
+
+/**
+ * This utility function returns a 
+ * "reasonable" set of points that bracket
+ * and divide an interval
+ */
+QList<double> GetTicksOuter(double amin, double amax, bool isLog, int requestedCounts);
+QList<double> GetTicksInner(double amin, double amax, bool isLog, int requestedCounts);
 
 //The HandleAxis class encapsulates a 3D axis object, and is
 //manipulated through the Set/Get interface.
