@@ -27,7 +27,7 @@ protected:
   QImage img;
   void PrepImageRGBNoAlphaMap(const double *dp, int rows, int cols,
 			      std::vector<double> &alpha);
-  double* RGBExpandImage(const double *dp, int rows, int cols);
+  double* RGBExpandImage(const double *dp, int rows, int cols, bool floatData);
   std::vector<double> GetAlphaMap(int rows, int cols);
 public:
   HandleImage();
