@@ -34,23 +34,24 @@ public:
   BaseFigureQt(QWidget *parent, HandleFigure *fig);
   void paintEvent(QPaintEvent *e);
   void resizeEvent(QResizeEvent *e);
-  //   QSize sizeHint() const;
-  //   QSizePolicy sizePolicy() const;
+  //  QSize sizeHint() const;
+  //  QSizePolicy sizePolicy() const;
 };
 
-// QSize BaseFigureQt::sizeHint() const {
-//   HPTwoVector *htv = (HPTwoVector*) hfig->LookupProperty("figsize");
-//   qDebug() << "Size hint " << (htv->Data()[0]) << "," << (htv->Data()[1]) << "\r\n";
-//   return QSize((int)(htv->Data()[0]),(int)(htv->Data()[1]));
+//  QSize BaseFigureQt::sizeHint() const {
+//    HPTwoVector *htv = (HPTwoVector*) hfig->LookupProperty("figsize");
+//    qDebug() << "Size hint " << (htv->Data()[0]) << "," << (htv->Data()[1]) << "\r\n";
+//    return QSize((int)(htv->Data()[0]),(int)(htv->Data()[1]));
 //   //  return QSize(10000,10000);
-// }
+//  }
 
 void BaseFigureQt::resizeEvent(QResizeEvent *e) {
   QWidget::resizeEvent(e);
   //  qDebug() << "resize " << width() << " " << height() << "\r\n";
+  backStore = QPixmap(qMax(8,width()),
+		      qMax(8,height()));
   hfig->resizeGL(qMax(8,width()),
   		 qMax(8,height()));
-  backStore = QPixmap(width(),height());
 }
 
 static bool enableRepaint = false;
@@ -65,10 +66,12 @@ void GfxDisableRepaint() {
 }
 
 void BaseFigureQt::paintEvent(QPaintEvent *e) {
+  //  qDebug() << "Paint\r";
   if (enableRepaint && hfig->ParentWindow()->isDirty()) {
     {
       // enableRepaint is true, and the background is dirty - update
       // the backing store, and then redraw it.
+      //      qDebug() << "Redraw\r";
       QPainter pnt(&backStore);
       QTRenderEngine gc(&pnt,0,0,width(),height());
       hfig->PaintMe(gc);
@@ -77,6 +80,7 @@ void BaseFigureQt::paintEvent(QPaintEvent *e) {
     QPainter pnt2(this);
     pnt2.drawPixmap(0,0,backStore);
   } else {
+    //    qDebug() << "Refresh\r";
     QPainter pnt(this);
     pnt.drawPixmap(e->rect(),backStore);
   }
@@ -179,6 +183,7 @@ HandleWindow::HandleWindow(unsigned ahandle) : QMainWindow() {
   setWindowTitle(buffer);
   qtchild = new BaseFigureQt(this,hfig);
   band = NULL;
+  setMinimumSize(50,50);
   //   if (QGLFormat::hasOpenGL())
   //     glchild = new BaseFigureGL(NULL,hfig);
   //  layout = new QStackedWidget(this);
@@ -203,7 +208,6 @@ HandleWindow::HandleWindow(unsigned ahandle) : QMainWindow() {
   dirty = false;
 }
 
-
 // Useful tools:
 //
 // Zoom in - fix for images
@@ -448,6 +452,7 @@ void HandleWindow::mouseMoveEvent(QMouseEvent* e) {
 	h->SetConstrainedStringDefault("ylimmode","manual");
 	h->UpdateState();
 	hfig->Repaint();
+	UpdateState();
       }
     }
     if ((mode == rotate_mode) && rotate_active) {
@@ -486,6 +491,7 @@ void HandleWindow::mouseMoveEvent(QMouseEvent* e) {
 	h->SetConstrainedStringDefault("cameraupvectormode","manual");
 	h->UpdateState();
 	hfig->Repaint();
+	UpdateState();
       }
     }
     if ((mode == cam_rotate_mode) && rotate_active) {
@@ -503,6 +509,7 @@ void HandleWindow::mouseMoveEvent(QMouseEvent* e) {
 	h->SetConstrainedStringDefault("cameraupvectormode","manual");
 	h->UpdateState();
 	hfig->Repaint();
+	UpdateState();
       }
     }
   } catch (Exception &e) {
@@ -613,11 +620,16 @@ void HandleWindow::mouseReleaseEvent(QMouseEvent * e) {
     }
   } catch (Exception &e) {
   }
+  dirty = true;
+}
+
+void HandleWindow::markClean() {
+  dirty = false;
 }
 
 void HandleWindow::UpdateState() {
   if (!initialized) return;
-    dirty = true;
+  dirty = true;
   //  HPTwoVector *htv = (HPTwoVector*) hfig->LookupProperty("figsize");
   //  qtchild->resize((int)(htv->Data()[0]),(int)(htv->Data()[1]));
   //  qtchild->updateGeometry();
@@ -640,6 +652,6 @@ void HandleWindow::UpdateState() {
   // 	glchild->setGeometry(0,0,1,1);
   //  layout->setCurrentWidget(qtchild);
   //     }
-    update();
+    repaint();
   //   }
 }
