@@ -65,7 +65,7 @@ KeyManager::KeyManager()  {
   ncolumn = 80;
   nline = 24;
   insert = true;
-  history.push_back("");
+  //  history.push_back("");
   enteredLinesEmpty = true;
   ReplacePrompt("");
   loopactive = 0;
@@ -74,6 +74,8 @@ KeyManager::KeyManager()  {
   context = NULL;
   QSettings settings("FreeMat", "FreeMat");
   QStringList historyList = settings.value("interpreter/history").toStringList();
+  if (!historyList.size())
+    history.push_back("");
   for (int i=0;i<historyList.size();i++) 
     history.push_back(historyList[i].toStdString());
 }
@@ -90,6 +92,10 @@ int KeyManager::getTerminalWidth() {
   return ncolumn;
 }
 
+void KeyManager::ClearHistory() {
+  history.clear();
+}
+
 void KeyManager::SetTermWidth(int w) {
   emit UpdateTermWidth(w);
   ncolumn = w;
@@ -613,8 +619,10 @@ void KeyManager::KillLine() {
 }
 
 void KeyManager::HistorySearchBackward() {
-  if (last_search != keyseq_count-1)
+  if (last_search != keyseq_count-1) {
     SearchPrefix(string(lineData),buff_curpos);
+    startsearch = history.size();
+  }
   last_search = keyseq_count;
   HistoryFindBackwards();
   ntotal = lineData.size();
@@ -641,7 +649,12 @@ void KeyManager::SearchPrefix(string aline, int aprefix_len) {
 void KeyManager::AddHistory(string mline) {
   prefix = "";
   prefix_len = 0;
-  history.push_back(mline);
+  if (mline.size() > 0) {
+    if (history.back() != mline)
+      history.push_back(mline);
+    while (history.size() > 1000)
+      history.pop_front();
+  }
   emit SendCommand(QString::fromStdString(mline));
   return;
 }
@@ -649,7 +662,12 @@ void KeyManager::AddHistory(string mline) {
 void KeyManager::HistoryFindForwards() {
   unsigned int i;
   bool found;
-  if (startsearch == 0) return;
+  if (startsearch >= (history.size()-1)) {
+    ResetLineBuffer();
+    AddStringToLine(prefix);
+    startsearch = history.size();
+    return;
+  }
   i = startsearch+1;
   found = false;
   while (i<history.size() && !found) {
@@ -658,7 +676,9 @@ void KeyManager::HistoryFindForwards() {
     if (!found) i++;
   }
   if (!found && (i >= history.size())) {
+    startsearch = history.size();
     ResetLineBuffer();
+    AddStringToLine(prefix);
     return;
   }
   lineData = history[i];
@@ -1013,6 +1033,9 @@ void KeyManager::QueueString(QString t) {
 }
 
 void KeyManager::QueueMultiString(QString t) {
+  t.replace(QChar(8233),"\n");
+  t.replace("\r\n","\n");
+  t.replace("\r","\n"); 
   if (t.indexOf("\n") < 0) {
     QueueString(t);
     return;
@@ -1072,7 +1095,8 @@ void KeyManager::RegisterTerm(QObject* term) {
   connect(this,SIGNAL(ClearEOL()),term,SLOT(ClearEOL()));
   connect(this,SIGNAL(ClearEOD()),term,SLOT(ClearEOD()));
   connect(this,SIGNAL(MoveBOL()),term,SLOT(MoveBOL()));
-  connect(this,SIGNAL(OutputRawString(string)),term,SLOT(OutputRawString(string)));
+  connect(this,SIGNAL(OutputRawString(string)),term,
+	  SLOT(OutputRawString(string)));
   connect(this,SIGNAL(ClearDisplay()),term,SLOT(ClearDisplay()));
   connect(term,SIGNAL(OnChar(int)),this,SLOT(OnChar(int)));
   connect(term,SIGNAL(SetTextWidth(int)),this,SLOT(SetTermWidth(int)));  
@@ -1179,12 +1203,22 @@ vector<string> KeyManager::GetCompletions(string line,
   if (!context) return completions;
   if (start[-1] != '\'') {
     vector<string> local_completions(context->getCompletions(string(start)));
-    for (int i=0;i<local_completions.size();i++)
-      completions.push_back(local_completions[i]);
+    for (int i=0;i<local_completions.size();i++) 
+      if (local_completions[i].find("private:") == local_completions[i].npos)
+	completions.push_back(local_completions[i]);
   }
   stringVector comp(GetCompletionList(tmp));
-  for (int i=0;i<comp.size();i++)
-    completions.push_back(comp[i]);
+  for (int i=0;i<comp.size();i++) 
+    if (comp[i].find("private:") == comp[i].npos)
+      completions.push_back(comp[i]);
   sort(completions.begin(),completions.end());
   return completions;
 }
+
+void KeyManager::WriteHistory() {
+  QSettings settings("FreeMat","FreeMat");
+  QStringList historyList;
+  for (int i=0;i<history.size();i++) 
+    historyList << QString::fromStdString(history[i]);
+  settings.setValue("interpreter/history",historyList);
+}
