@@ -86,7 +86,7 @@ MainApp::MainApp() {
   GUIHack = false;
   skipGreeting = false;
   m_keys = new KeyManager;
-  m_global = new Scope("global");
+  m_global = new Scope("global",false);
   // The global scope is special
   m_global->mutexSetup();
 }
@@ -100,11 +100,16 @@ void MainApp::HelpWin() {
 }
 
 void MainApp::SetupGUICase() {
+  if (inBundleMode()) {
+    QDir dir(QApplication::applicationDirPath());
+    dir.cdUp();
+    dir.cd("Plugins");
+    QString dummy(dir.absolutePath());
+    QApplication::setLibraryPaths(QStringList() << dir.absolutePath());
+  }
   m_win = new ApplicationWindow;
-  QTTerm *gui = new QTTerm(NULL);
+  QTTerm *gui = new QTTerm;
   m_keys->RegisterTerm(gui);
-  gui->resizeTextSurface();
-  //  gui->show();
   m_win->SetGUITerminal(gui);
   m_win->SetKeyManager(m_keys);
   m_win->readSettings();
@@ -169,6 +174,34 @@ void MainApp::PathTool() {
   PathToolFunction(0,dummy,m_eval);
 }
 
+extern MainApp *m_app;
+
+
+//!
+//@Module EDITOR Open Editor Window
+//@@Section FREEMAT
+//@@Usage
+//Brings up the editor window.  The @|editor| function takes no
+//arguments:
+//@[
+//  editor
+//@]
+//!
+ArrayVector EditorFunction(int nargout, const ArrayVector& arg, Interpreter* eval) {
+  static FMEditor *edit = NULL;
+  if (edit == NULL) {
+    edit = new FMEditor(eval);
+    QObject::connect(eval,SIGNAL(RefreshBPLists()),edit,SLOT(RefreshBPLists()));
+    QObject::connect(eval,SIGNAL(ShowActiveLine()),edit,SLOT(ShowActiveLine()));
+    // Because of the threading setup, we need the keymanager to relay commands
+    // from the editor to the interpreter.  
+    QObject::connect(edit,SIGNAL(EvaluateText(QString)),m_app->GetKeyManager(),SLOT(QueueMultiString(QString)));
+  }
+  edit->showNormal();
+  edit->raise();
+  return ArrayVector();
+}
+
 void MainApp::Editor() {
   ArrayVector dummy;
   EditorFunction(0,dummy,m_eval);
@@ -193,6 +226,7 @@ void MainApp::Crashed() {
 
 void MainApp::Quit() {
   TerminalReset();
+  m_keys->WriteHistory();
   qApp->closeAllWindows();
   qApp->quit();
 }
@@ -228,6 +262,7 @@ void MainApp::CheckNonClosable() {
     wid = new NonClosable;
     wid->setGeometry(0,0,1,1);
     wid->setWindowIcon(QIcon(":/images/freemat_small_mod_64.png"));
+    wid->setWindowTitle("FreeMat v" VERSION);
     wid->show();
   }
 }
@@ -268,7 +303,6 @@ ArrayVector ThreadIDFunction(int nargout, const ArrayVector& arg, Interpreter* e
   return ArrayVector() << Array::uint32Constructor(eval->getThreadID());
 }
 
-extern MainApp *m_app;
 //!
 //@Module PAUSE Pause Script Execution
 //@@Section IO
@@ -736,6 +770,7 @@ ArrayVector ClcFunction(int nargout, const ArrayVector& arg) {
   return ArrayVector();
 }
 
+
 void LoadThreadFunctions(Context *context) {
   context->addSpecialFunction("threadid",ThreadIDFunction,0,1,NULL);
   context->addSpecialFunction("threadnew",ThreadNewFunction,0,1,NULL);
@@ -747,6 +782,7 @@ void LoadThreadFunctions(Context *context) {
   context->addGfxSpecialFunction("pause",PauseFunction,1,0,"x",NULL);
   context->addGfxSpecialFunction("sleep",SleepFunction,1,0,"x",NULL);
   context->addGfxFunction("clc",ClcFunction,0,0,NULL);
+  context->addGfxSpecialFunction("editor",EditorFunction,0,0,NULL);
 }
 
 void MainApp::EnableRepaint() {
@@ -775,11 +811,6 @@ void MainApp::UpdatePaths() {
   static bool paths_set = false;
   if (!paths_set) {
     if (inBundleMode()) {
-      QDir dir(QApplication::applicationDirPath());
-      dir.cdUp();
-      dir.cd("Plugins");
-      QString dummy(dir.absolutePath());
-      QApplication::addLibraryPath(dir.absolutePath());
       QDir dir1(qApp->applicationDirPath() + "/../Resources/toolbox");
       if (dir1.exists()) {
 	QString path1(dir1.canonicalPath());
